//  This file is really -*- c++ -*-
//  file: /C++/stopwatch.c++
//  Implementation of the stopwatch class.

#include "stopwatch.h"
#include <sys/time.h>
#include <iostream>
#include <iomanip>

// WARNING: do NOT try to compile with -ansi.  This is not ANSI compliant.
// The problem is that hrtime_t is long long which is not ansi compliant.

// copyright 2008 David Casperson


class StopWatchImplementation
{
private:
    typedef hrtime_t  stopwatch_t;

public:
    StopWatchImplementation () ;
    // The three buttons.
    StopWatchImplementation& start() ;
    StopWatchImplementation& stop() ;
    StopWatchImplementation& reset() ;

    // and queries
    double elapsed() const ;
    bool is_running() const ;
private:
    void start_click() ;
    void end_click() ;
    stopwatch_t last_button_time ;
    // last time at which start() or reset() or stop() was pressed.
    
    double total ;              // total elapsed time between the last
                                // press of reset() and the last press
                                // of stop().
    
    bool running ;               // true iff start() was pressed more
                                // recently than stop().

} ;



StopWatch::StopWatch()
    : it(new StopWatchImplementation())
{}

StopWatch::~StopWatch()
{
    delete it ;
    it = 0 ;
    return ;
}

StopWatch& StopWatch::start() 
{
    it->start() ;
    return *this ;
}

StopWatch& StopWatch::stop()
{
    it->stop() ;
    return *this ;
}

StopWatch& StopWatch::reset()
{
    it->reset() ;
    return *this ;
}

double StopWatch::elapsed() const
{
    return it->elapsed() ;
}

bool
StopWatch::is_running() const
{
    return it->is_running() ;
}

std::ostream& operator<< (std::ostream& o, const StopWatch& s) 
{
    o << std::setprecision(3) << s.elapsed() << "s " ;
    return o ;
}

      

StopWatchImplementation::StopWatchImplementation()
{
    stop() ;
    reset() ;
}

void
StopWatchImplementation::start_click()
{
    stopwatch_t time2 = gethrvtime() ;
    if (running)
        total += static_cast<double>(time2-last_button_time)/ 1.0E9 ;
}

void
StopWatchImplementation::end_click()
{
    last_button_time = gethrvtime() ;
}

StopWatchImplementation& StopWatchImplementation::start() 
{
    start_click() ;
    running = true ;
    end_click() ;
    return *this ;
}

StopWatchImplementation& StopWatchImplementation::stop()
{
    start_click() ;
    running = false ;
    end_click() ;
    return *this ;
}

StopWatchImplementation& StopWatchImplementation::reset()
{
    start_click() ;
    total = 0.0 ;
    end_click() ;
    return *this ;
}

double StopWatchImplementation::elapsed() const
{
    stopwatch_t  now = gethrvtime() ;
    double extra = (running ?
                    static_cast<double>(now-last_button_time) :
                    0.0 )
        / 1.0E9 ;
    return total + extra ;
}

bool
StopWatchImplementation::is_running() const
{
    return running ;
}
